/*
 * mokios.c -- example of a program using fork()
 *
 * Copyright (C) 2006 Argiris Mokios <amok@auth.gr>
 * Copyright (C) 2006 Giorgos Keramidas <keramida@ceid.upatras.gr>
 *
 * Run a child process using argv[1] ... argv[argc-1], wait for it to
 * complete, and print resource usage with getrusage() when it ends.
 */

#include <sys/types.h>

#include <sys/resource.h>
#include <sys/time.h>
#include <sys/wait.h>

#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <unistd.h>

static void usage(void);
static int execute(char *av[]);

int
main(int argc, char *argv[])
{
	if (argc < 2)
		usage();

	if (execute(argv + 1) < 0)
		exit(EXIT_FAILURE);
	return EXIT_SUCCESS;
}

static void
usage(void)
{
	fprintf(stderr, "usage: mokios cmd [arg ...]\n");
	exit(EXIT_FAILURE);
}

static int
execute(char *av[])
{
	struct rusage rbuf;
	pid_t pid;
	int status;

	assert(av != NULL);

	if ((pid = fork()) == -1) {
		perror("fork");
		return -1;
	}

	/*
	 * The child executes the command we are supposed to run, or it
	 * terminates with an error code.
	 */
	if (pid == 0) {
		execvp(*av, av);
		/*
		 * The execvp() call should not return to us again.  If it
		 * does, it's an operating system error, so let our parent
		 * know we failed.
		 */
		_exit(EX_OSERR);
	}

	/*
	 * The parent process waits for the child to finish, and gather its
	 * resource usage statistics.
	 */

	if (waitpid(pid, &status, 0) != pid) {
		perror("waitpid");
		return -1;
	}
	if ((getrusage(RUSAGE_CHILDREN, &rbuf)) == -1) {
		perror("getrusage");
		return -1;
	}

	printf("pid %llu\n", (unsigned long long)pid);
	printf("  user time used (utime) = %llu.%06llu\n",
	    (unsigned long long)rbuf.ru_utime.tv_sec,
	    (unsigned long long)rbuf.ru_utime.tv_usec);
	printf("  system time used (stime) = %llu.%06llu\n",
	    (unsigned long long)rbuf.ru_stime.tv_sec,
	    (unsigned long long)rbuf.ru_stime.tv_usec);
	printf("  max resident set size (maxrss) = %lu\n", rbuf.ru_maxrss);
	printf("  integral shared text memory size (ixrss) = %lu\n", rbuf.ru_ixrss);
	printf("  integral unshared data size (idrss) = %lu\n", rbuf.ru_idrss);
	printf("  integral unshared stack size (isrss) = %lu\n", rbuf.ru_isrss);
	printf("  page reclaims (minflt) = %lu\n", rbuf.ru_minflt);
	printf("  page faults (majflt) = %lu\n", rbuf.ru_majflt);
	printf("  swaps (nswap) = %lu\n", rbuf.ru_nswap);
	printf("  block input operations (inblock) = %lu\n", rbuf.ru_inblock);
	printf("  block output operations (oublock) = %lu\n", rbuf.ru_oublock);
	printf("  messages sent (msgsnd) = %lu\n", rbuf.ru_msgsnd);
	printf("  messages received (msgrcv) = %lu\n", rbuf.ru_msgrcv);
	printf("  signals received (nsignals) = %lu\n", rbuf.ru_nsignals);
	printf("  voluntary context switches (nvcsw) = %lu\n", rbuf.ru_nvcsw);
	printf("  involuntary context switches (nivcsw) = %lu\n", rbuf.ru_nivcsw);

	return 0;
}
